#!/usr/bin/env python

# Python module imports.
import argparse, sys, MDAnalysis, math, numpy

###
### Input / Choose the number of structures/dipoles
###

parser = argparse.ArgumentParser(description='Roll lattice into cylinder')
parser.add_argument('Unit_cell', metavar='foo', type=str, help='Name of the unit cell')
parser.add_argument('Height', metavar='foo', type=int, help='Height of the lattice (No unit cells)')
parser.add_argument('Width', metavar='foo', type=int, help='Width of the lattice (No unit cells)')
parser.add_argument('Angle', metavar='foo', type=float, help='Angle of rotation')
parser.add_argument('Radius', metavar='foo', type=float, help='Radius of the cylinder (nm)')
# parser.add_argument('Output', metavar='foo', type=str, help='Output cylinder')

args = parser.parse_args()

#########
######### EXAMPLE: ./create_cylinder_v2.0.py template.pdb 20 30 45.0 5.0
#########


###
### Create lattice
###

row = [MDAnalysis.Universe(args.Unit_cell) for i in range(0,args.Width)]	# create list of universes for the first row

u = MDAnalysis.Universe(args.Unit_cell)		#
[x,y,z,a,b,c] = u.dimensions			# get the features of the unit cell
del u						#

for resid, cell in enumerate(row):					# create the first row by 
    cell.atoms.rotateby(b-90,[0,1,0],[0,0,0])				# rotating each box to match the cartesian coordinates
    cell.atoms.positions = cell.atoms.positions + (0,0,z*resid)		# and translating the universe

big_row = MDAnalysis.Merge(*[cell.atoms for cell in row])		# merge universes and save 
big_row.atoms.write("row.pdb")						#


stacks = [MDAnalysis.Universe("row.pdb") for i in range(0,args.Height)]		# create list of universes for each stack of rows

for resid, stack in enumerate(stacks):					# translate each stack universe
    stack.atoms.positions = stack.atoms.positions + (0,y*resid,0)	#

lattice = MDAnalysis.Merge(*[stack.atoms for stack in stacks])		# merge universes and save 
lattice.atoms.write("lattice.pdb")					#

###
### Rotate lattice
###

sina = numpy.sin(args.Angle*math.pi/180)	# calculate sin and cos for the rotation angle		
cosa = numpy.cos(args.Angle*math.pi/180)	#

pos = lattice.atoms.positions						# 
posx = pos[:,0]								# extract and modify lattice coordinates 
posy = cosa*pos[:,1]-sina*pos[:,2]					#
posz = sina*pos[:,1]+cosa*pos[:,2]					#
lattice.atoms.positions = numpy.vstack((posx,posy,posz)).T.tolist()	# replace lattice positions

lattice.atoms.write("lattice_rot.pdb")

###
### Roll lattice
###
zcos = numpy.cos(lattice.atoms.positions[:,2]*1/(args.Radius*10))				# 
zsin = numpy.sin(lattice.atoms.positions[:,2]*1/(args.Radius*10))				# modify lattice coordinates
lattice.atoms.positions = lattice.atoms.positions - (lattice.atoms.positions[12,0].mean(),0,0)	# 
lattice.atoms.positions = lattice.atoms.positions + ((args.Radius*10),0,0)			# 

pos = lattice.atoms.positions
posx = pos[:,0]*zcos
posy = pos[:,1]								# extract and modify lattice coordinates
posz = pos[:,0]*zsin
lattice.atoms.positions = numpy.vstack((posx,posy,posz)).T.tolist()

lattice.atoms.write("cylinder_"+str(args.Width)+"_"+str(args.Angle)+"_"+str(args.Radius)+"_.pdb")	# save

